#include "GlewContentManagement.h"

using namespace std;


GlewContentManagement* GlewContentManagement::instance = NULL;
bool GlewContentManagement::initialized = false;

GlewContentManagement::~GlewContentManagement()
{
	if(m_GlewRenderWindowContext)
		delete m_GlewRenderWindowContext;
	if(m_GlewBottomWindowContext)
		delete m_GlewBottomWindowContext;
	if(m_GlewRightWindowContext)
		delete m_GlewRightWindowContext;
	if(m_GlewMainWindowContext)
		delete m_GlewMainWindowContext;
}

GlewContentManagement::GlewContentManagement()
{
	m_GlewRenderWindowContext = NULL;
	m_GlewBottomWindowContext = NULL;
	m_GlewRightWindowContext = NULL;
	m_GlewMainWindowContext = NULL;

	m_RenderWindowID = 0;
	m_BottomWindowID = 0;
	m_RightWindowID = 0;
	m_MainWindowID = 0;
}

GlewContentManagement *GlewContentManagement::getInstance()
{
	if(!initialized)
	{
		instance = new GlewContentManagement();
		initialized = true;
	}

	return instance;
}

void GlewContentManagement::initRenderWindowContext()
{
	m_RenderWindowID = glutGetWindow();
	m_GlewRenderWindowContext = new GLEWContext();

	initGlew();
}

void GlewContentManagement::initBottomWindowContext()
{
	m_BottomWindowID = glutGetWindow();
	m_GlewBottomWindowContext = new GLEWContext();

	initGlew();
}

void GlewContentManagement::initRightWindowContext()
{
	m_RightWindowID = glutGetWindow();
	m_GlewRightWindowContext = new GLEWContext();

	initGlew();
}

void GlewContentManagement::initMainWindowContext()
{
	m_MainWindowID = glutGetWindow();
	m_GlewMainWindowContext = new GLEWContext();

	initGlew();
}


GLEWContext* GlewContentManagement::getCurrentGlewContent()
{
	if(glutGetWindow() == m_RenderWindowID) 
		return m_GlewRenderWindowContext;
	if(glutGetWindow() == m_BottomWindowID) 
		return m_GlewBottomWindowContext;
	if(glutGetWindow() == m_RightWindowID) 
		return m_GlewRightWindowContext;
	if(glutGetWindow() == m_MainWindowID) 
		return m_GlewMainWindowContext;

	cout << "Failed to find the correct GLEWContext associated with the current glut window" << endl;
	return m_GlewRenderWindowContext;
}

void GlewContentManagement::initGlew()
{
	// Initialize GLEW
	std::cout << "- Initializing GLEW for ";

	if(glutGetWindow() == m_RenderWindowID)
		std::cout << "the main display window";
	if(glutGetWindow() == m_BottomWindowID)
		std::cout << "the bottom window";
	if(glutGetWindow() == m_RightWindowID)
		std::cout << "the right window";
	if(glutGetWindow() == m_MainWindowID)
		std::cout << "the main window";

	std::cout << "..." << std::endl;

	GLenum err = glewInit();

	if (err != GLEW_OK)
	{
		// glewInit failed, something is seriously wrong
		std::cerr << "Error initializing GLEW: " << glewGetErrorString(err) << std::endl;
	}

	fprintf(stdout, "- GLEW initialized (OpenGL  %s, GLSL %s) \n\n", glGetString(GL_VERSION), glGetString(GL_SHADING_LANGUAGE_VERSION));
}



GLEWContext* glewGetContext()
{
	GlewContentManagement* glewContentManager = GlewContentManagement::getInstance();

	return glewContentManager->getCurrentGlewContent();
}