#include "HistogramDraw.h"

#include "TransferFunctionGUI.h"
#include "Volume.h"

HistogramDraw* HistogramDraw::_instance = NULL;
bool HistogramDraw::initialized = false;

HistogramDraw::~HistogramDraw()
{
	if(m_HistogramData)
		delete m_HistogramData;
	if(m_HistogramDataLogarithmic)
		delete m_HistogramData;
}

HistogramDraw::HistogramDraw()
{
	m_HistogramData = NULL;
	m_HistogramDataLogarithmic = NULL;
	m_HistogramDataSize = 0;
	m_HistogramDrawType = HISTOGRAM_DRAW_LOG;
}

HistogramDraw *HistogramDraw::getInstance()
{
	if(!initialized)
	{
		_instance = new HistogramDraw();
		initialized = true;
	}

	return _instance;
}




void HistogramDraw::refreshHistogramDrawData(Volume& volume)
{
	if(m_HistogramData)
		delete m_HistogramData;
	if(m_HistogramDataLogarithmic)
		delete m_HistogramDataLogarithmic;

	std::vector<double> normalizedHistogramData = volume.GetNormHistogram();

	m_HistogramDataSize = normalizedHistogramData.size();
	m_HistogramData = new float[m_HistogramDataSize];
	m_HistogramDataLogarithmic = new float[m_HistogramDataSize];

	rescaleHistogramDataToHighestValue(normalizedHistogramData);
	calculateLogarithmicHistogramData();
}


void HistogramDraw::draw()
{
	if(!m_HistogramDrawType == HISTOGRAM_DRAW_NONE)
	{
		int histogramDrawWidth = TransferFunctionGUI::g_iTransferFuncPickerWidth;

		glLineWidth(1.0f);
		glEnable(GL_LINE_SMOOTH);
		glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);


		glBegin(GL_LINES);
		glColor4f(0.3f,0.4f,0.4f,0.3f);

		for(int i = 0; i < m_HistogramDataSize; ++i)
		{
			float yFactor;

			if(m_HistogramDrawType == HISTOGRAM_DRAW_LINEAR)
				yFactor = m_HistogramData[i];
			else if(m_HistogramDrawType == HISTOGRAM_DRAW_LOG)
				yFactor = m_HistogramDataLogarithmic[i];

			float y = TransferFunctionGUI::g_iTransferFuncPickerHeight * yFactor;
			float x = histogramDrawWidth *  i / (float) m_HistogramDataSize;

			glVertex2f(x, - y);
			glVertex2f(x, 0.f);
		}


		glEnd();

		glDisable(GL_LINE_SMOOTH);
	}
}


void HistogramDraw::setDrawType(HistogramDrawType drawType)
{
	m_HistogramDrawType = drawType;
}

void HistogramDraw::rescaleHistogramDataToHighestValue( std::vector<double> &normalizedHistogramData )
{
	float histoMax = 0.f;
	for(unsigned int i = 0; i < normalizedHistogramData.size(); ++i)
	{
		if(normalizedHistogramData.at(i) > histoMax)
			histoMax = (float)normalizedHistogramData.at(i);
	}


	for(unsigned int i = 0; i < normalizedHistogramData.size(); ++i)
	{
		m_HistogramData[i] = (float)normalizedHistogramData.at(i) / histoMax;
	}
}

void HistogramDraw::calculateLogarithmicHistogramData()
{
	for(int i = 0; i < m_HistogramDataSize; ++i)
	{
		m_HistogramDataLogarithmic[i] = 1.f - pow(10, ( - m_HistogramData[i] * 2.f));
	}
}