//Includes
#include "TFControlPoint.h"

#include "math.h"
#include <iostream>

#include "TransferFunctionGUI.h"

using namespace std;
using namespace Colours;


bool TFControlPoint::g_bMouseInsideBottomWindow = false;

int TFControlPoint::MAXIMUM_CONTROLPOINT_OUTSIDE_DRAG_DIST_SQUARED = 289;


TFControlPoint::TFControlPoint(float x, float y, const Lab_Colour& colour)
:
m_colour(colour)
{
	m_pos.x = x;
	m_pos.y = y;

	beingDragged = false;
	selected = false;
	markedForDeletion = false;

	dragDifferenceX = 0;
	dragDifferenceY = 0;
}


bool TFControlPoint::operator == (const TFControlPoint& other)
{
	return (this->m_pos.x == other.m_pos.x);
}

bool TFControlPoint::operator < (const TFControlPoint& other)
{
	return (this->m_pos.x < other.m_pos.x);
}



void TFControlPoint::setPosition(float x,  float y)
{
	m_pos.x = x;
	m_pos.y = y;
}

void TFControlPoint::setColour(float l, float a, float b)
{
	m_colour.L = l;
	m_colour.a = a;
	m_colour.b = b;
}

Lab_Colour& TFControlPoint::getColourLab()
{
	return m_colour;
}


void TFControlPoint::drawControlPoint()
{
	float radius = 4.f;
	float square = 4*sqrt(radius + 2.f);

	RGBA_Colour colourRGBA(toRGB(m_colour), GLubyte(255));

	float x = m_pos.x * TransferFunctionGUI::g_iTransferFuncPickerWidth;
	float y = -m_pos.y * TransferFunctionGUI::g_iTransferFuncPickerHeight;



	//Draw control point at origin
	glPushMatrix();
	glTranslatef( x, y, 0.f );

	drawControlPointNormal(square, colourRGBA, radius);

	glPopMatrix();


	if(beingDragged)
	{
		//Draw control point at moved position
		float movePosX = x - dragDifferenceX;
		float movePosY = y - dragDifferenceY;
		clamp(movePosX, 0.f, TransferFunctionGUI::g_iTransferFuncPickerWidth);
		clamp(movePosY, -TransferFunctionGUI::g_iTransferFuncPickerHeight, 0);

		glPushMatrix();
		glTranslatef((float)movePosX, (float)movePosY, 0.f );

		drawControlPointDragged(square, colourRGBA, radius);

		glPopMatrix();
	}
}



void TFControlPoint::drawControlPointCenterPoint( RGBA_Colour &colourRGBA, float radius, bool drawVersionDragged )
{
	glEnable(GL_LINE_SMOOTH);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);

	glBegin(GL_TRIANGLE_FAN);

	
	if(beingDragged && !drawVersionDragged)
		glColor4ub(colourRGBA.r, colourRGBA.g, colourRGBA.b, (GLubyte)(85));
	else
		glColor4ub(colourRGBA.r, colourRGBA.g, colourRGBA.b, (GLubyte)(255));

	glVertex2f(0, 0);
	for(float angle = 0.; angle < 360.f; angle += 5.f)
	{
		glVertex2f(sin(angle) * radius, cos(angle) * radius);
	}
	glEnd();

	glDisable(GL_LINE_SMOOTH);
}

void TFControlPoint::drawControlPointFrame(float square, bool drawDraggedVersion, bool drawVersionDeleting)
{
	RGBA_Colour frameColour;

	if(beingDragged && !drawDraggedVersion)
		frameColour.setColour(0, 0, 0, 85);
	else
		frameColour.setColour(0, 0, 0, 255);

	if(selected)
	{
		frameColour.r = (GLubyte)255;
		frameColour.g = (GLubyte)255;
		frameColour.b = (GLubyte)0;
	}
	
	if(drawVersionDeleting)
	{
		frameColour.r = (GLubyte)255;
		frameColour.g = (GLubyte)0;
		frameColour.b = (GLubyte)0;
	}

	glLineWidth(1.5f); 
	glBegin(GL_LINE_LOOP);
	glColor4ub(frameColour.r, frameColour.g, frameColour.b, frameColour.a);
	glVertex2f(-square, 0.f);
	glVertex2f(0.f, +square);
	glVertex2f(square, 0.f);
	glVertex2f(0.f, -square);
	glColor3i(0,0,0);
	glEnd();
}

void TFControlPoint::drawControlPointBackGround( float square, bool drawDraggedVersion)
{
	GLubyte transparency;

	if(beingDragged && !drawDraggedVersion)
		transparency = (GLubyte)127;
	else
		transparency = (GLubyte)42;
		
	glBegin(GL_TRIANGLES);
	glColor4ub(255, 255, 255, 127);
	glVertex2f(0.f, 0.f);
	glVertex2f(0.f, square);
	glVertex2f(square, 0.f);
	glVertex2f(0.f, 0.f);
	glVertex2f(0.f, -square);
	glVertex2f(-square, 0.f);
	glColor4ub(0, 0, 0, 127);
	glVertex2f(0.f, 0.f);
	glVertex2f(0.f, -square);
	glVertex2f(square, 0.f);
	glVertex2f(0.f, 0.f);
	glVertex2f(0.f, square);
	glVertex2f(-square, 0.f);
	glEnd();
}

void TFControlPoint::setBeingDragged(bool beingDragged)
{
	this->beingDragged = beingDragged;
}

bool TFControlPoint::isBeingDragged()
{
	return beingDragged;
}


void TFControlPoint::setSelected(bool selected)
{
	this->selected = selected;
}

bool TFControlPoint::isSelected()
{
	return selected;
}

void TFControlPoint::dragMovement(int mouseX, int mouseY)
{
	updateDragDifference(mouseX, mouseY);

	updateBeingDraggedIntoDeletion(mouseX, mouseY);
}

void TFControlPoint::draggingEnded()
{
	this->beingDragged = false;
	this->markedForDeletion = false;
	m_pos.x -= dragDifferenceX / TransferFunctionGUI::g_iTransferFuncPickerWidth;
	m_pos.y += dragDifferenceY / TransferFunctionGUI::g_iTransferFuncPickerHeight;

	clamp(m_pos.x, 0.f, 1.f);
	clamp(m_pos.y, 0.f, 1.f);
}

void TFControlPoint::updateDragDifference( int mouseX, int mouseY )
{
	dragDifferenceX = m_pos.x * TransferFunctionGUI::g_iTransferFuncPickerWidth - mouseX;
	dragDifferenceY = TransferFunctionGUI::g_iTransferFuncPickerHeight - m_pos.y * TransferFunctionGUI::g_iTransferFuncPickerHeight - mouseY;
	if(dragDifferenceX * dragDifferenceX + dragDifferenceY * dragDifferenceY < 33)
	{
		dragDifferenceX = 0;
		dragDifferenceY = 0;
	}
}

void TFControlPoint::updateBeingDraggedIntoDeletion( int mouseX, int mouseY )
{
	int movePosX = m_pos.x * TransferFunctionGUI::g_iTransferFuncPickerWidth - dragDifferenceX;
	int movePosY = - m_pos.y * TransferFunctionGUI::g_iTransferFuncPickerHeight - dragDifferenceY;
	int movePosXCopy = movePosX;
	int movePosYCopy = movePosY;
	clamp(movePosX, 0, TransferFunctionGUI::g_iTransferFuncPickerWidth);
	clamp(movePosY, -TransferFunctionGUI::g_iTransferFuncPickerHeight, 0);

	markedForDeletion =
		(pow(movePosXCopy - (float)movePosX, 2.f) + pow(movePosYCopy - (float)movePosY, 2.f)) > MAXIMUM_CONTROLPOINT_OUTSIDE_DRAG_DIST_SQUARED
		|| !g_bMouseInsideBottomWindow;
}

bool TFControlPoint::isMarkedForDeletion()
{
	return markedForDeletion;
}

void TFControlPoint::setMarkedForDeletion(bool deletePoint)
{
	markedForDeletion = deletePoint;
}

void TFControlPoint::drawControlPointNormal( float square, RGBA_Colour colourRGBA, float radius )
{
	drawControlPointBackGround(square);

	drawControlPointCenterPoint(colourRGBA, radius);

	drawControlPointFrame(square, false, markedForDeletion);
}

void TFControlPoint::drawControlPointDragged( float square, RGBA_Colour colourRGBA, float radius )
{
	drawControlPointBackGround(square, true);

	drawControlPointCenterPoint(colourRGBA, radius, true);

	drawControlPointFrame(square, true, markedForDeletion);
}