#ifndef TF_CONTROL_POINT_H
#define TF_CONTROL_POINT_H

#include "Colours.h"
#include "BasicObjects.h"

/**
* @class TFControlPoint
*
* @brief  Handles transfer function control points
*
*/

class TFControlPoint
{
public:
	TFControlPoint(float x, float y, const Lab_Colour& colour);

	bool operator == (const TFControlPoint& other);
	bool operator < (const TFControlPoint& other);

	void setPosition(float x, float y);
	void setColour(float l, float a, float b);

	bool isBeingDragged();
	void setBeingDragged(bool beingDragged);
	bool isSelected();
	void setSelected(bool selected);
	bool isMarkedForDeletion();
	void setMarkedForDeletion(bool deletePoint);

	Lab_Colour& getColourLab();
	void drawControlPoint();		//draw a cross

	void drawControlPointDragged( float square, RGBA_Colour colourRGBA, float radius );
	void drawControlPointNormal( float square, RGBA_Colour colourRGBA, float radius );
	void drawControlPointBackGround( float square, bool drawVersionDragged = false);
	void drawControlPointFrame(float squares, bool drawVersionDragged = false, bool drawVersionDeleting = false);
	void drawControlPointCenterPoint( RGBA_Colour &colourRGBA, float radius, bool drawVersionDragged = false );

	void dragMovement(int mouseX, int mouseY); 

	void updateBeingDraggedIntoDeletion( int mouseX, int mouseY );
	void updateDragDifference( int mouseX, int mouseY );
	void draggingEnded();

	Position	m_pos;
	float		dragDifferenceX;
	float		dragDifferenceY;
	Lab_Colour	m_colour;

	static const int	DrawSize = 10;
	static bool			g_bMouseInsideBottomWindow;

	static int			MAXIMUM_CONTROLPOINT_OUTSIDE_DRAG_DIST_SQUARED;

private:
	bool selected;
	bool beingDragged;
	bool markedForDeletion;
};

#endif