#include "TransferFunctionGUI.h"

#include "utility.h"

#include "Colours.h"
#include "TFControlPoint.h"

#include <algorithm>
#include <sstream>

#include "tinyxml.h"

using namespace Colours;

RGBA_Colour* TransferFunctionGUI::transferFunctionPickerData = new RGBA_Colour[g_iTransferFuncPickerHeight * g_iTransferFuncPickerWidth];

int TransferFunctionGUI::g_iTransferFunctionTextureDisplayHeight = 14;
int TransferFunctionGUI::g_iTransferFunctionTextureDisplayYOffset = 8;

std::vector<TFControlPoint> TransferFunctionGUI::controlPoints;

ColourSpace TransferFunctionGUI::transferFunctionTextureColorSpace = COLOURSPACE_LAB;

const int TransferFunctionGUI::g_iTransferFuncPickerWidth = 660;
const int TransferFunctionGUI::g_iTransferFuncPickerHeight = 180;



void TransferFunctionGUI::refreshTransferFuncPicker()
{
	//Special cases - No points
	if(controlPoints.size() == 0)
	{
		for(int x = 0; x < g_iTransferFuncPickerWidth; ++x)
			for(int y = 0; y < g_iTransferFuncPickerHeight; ++y)
				transferFunctionPickerData[(int)g_iTransferFuncPickerWidth * y + x] = RGBA_Colour((GLubyte)0, (GLubyte)0, (GLubyte)0, (GLubyte)0);
	}
	else
	{
		refreshTransferFuncPickerData();
	}
}

void TransferFunctionGUI::initControlPoints()
{
	Lab_Colour colour((GLubyte)255, (GLubyte)0, (GLubyte)0);
	controlPoints.push_back(TFControlPoint(0.17f, 0.f, colour));

	colour = Lab_Colour((GLubyte)127, (GLubyte)0, (GLubyte)67);
	controlPoints.push_back(TFControlPoint(0.75f, 99.f / g_iTransferFuncPickerHeight, colour));

	colour = Lab_Colour((GLubyte)127, (GLubyte)127, (GLubyte)0);
	controlPoints.push_back(TFControlPoint(0.26f, 180.f / g_iTransferFuncPickerHeight, colour));

	colour = Lab_Colour((GLubyte)55, (GLubyte)55, (GLubyte)0);
	controlPoints.push_back(TFControlPoint(1.f, 22.f / g_iTransferFuncPickerHeight, colour));

	sortControlPoints();
}

void TransferFunctionGUI::drawControlPointsRelative()
{
	if(controlPoints.size() > 0)
	{
		//Prepend unselected controlPoints
		for(std::vector<TFControlPoint>::iterator iter = controlPoints.begin(); iter != controlPoints.end(); ++iter)
		{
			TFControlPoint& controlPoint = *iter;

			if(!controlPoint.isSelected())
				controlPoint.drawControlPoint();
		}
		//Now draw selected controlPoints on top
		for(std::vector<TFControlPoint>::iterator iter = controlPoints.begin(); iter != controlPoints.end(); ++iter)
		{
			TFControlPoint& controlPoint = *iter;

			if(controlPoint.isSelected())
				controlPoint.drawControlPoint();
		}
	}
}


void TransferFunctionGUI::drawControlPointsConnection()
{
	glLineWidth(1.5f);
	glEnable(GL_LINE_SMOOTH);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);


	glBegin(GL_LINE_STRIP);
	glColor3f(0.f,0.f,0.f);
	glVertex2i(0, 0);
	if(controlPoints.size() > 0)
	{
		for(std::vector<TFControlPoint>::iterator iter = controlPoints.begin(); iter != controlPoints.end(); ++iter)
		{
			TFControlPoint& controlPoint = *iter;

			glVertex2f(controlPoint.m_pos.x * g_iTransferFuncPickerWidth ,-controlPoint.m_pos.y * g_iTransferFuncPickerHeight);
		}
	}
	glVertex2i(g_iTransferFuncPickerWidth, 0);
	glEnd();

	glDisable(GL_LINE_SMOOTH);
}


bool TransferFunctionGUI::AABBHitCheck(int mouseX, int mouseY, float AABBSideLength, float AABBCenterX, float AABBCenterY)
{
	return (mouseX >= AABBCenterX - AABBSideLength  && mouseX <= AABBCenterX + AABBSideLength
		&& mouseY >= AABBCenterY - AABBSideLength  && mouseY <= AABBCenterY + AABBSideLength);
}

void TransferFunctionGUI::setAllControlPointsUndragged()
{
	for(std::vector<TFControlPoint>::iterator iter = controlPoints.begin(); iter != controlPoints.end();)
	{
		TFControlPoint& controlPoint = *iter;

		if(controlPoint.isBeingDragged())
		{
			if(controlPoint.isMarkedForDeletion())
				iter = controlPoints.erase(iter);
			else
			{
				++iter;
				controlPoint.draggingEnded();
			}
		}
		else
			++iter;
	}
}

void TransferFunctionGUI::setAllControlPointsUnselected()
{
	for(std::vector<TFControlPoint>::iterator iter = controlPoints.begin(); iter != controlPoints.end(); ++iter)
	{
		TFControlPoint& controlPoint = *iter;

		controlPoint.setSelected(false);
	}
}

bool TransferFunctionGUI::isAnyControlPointBeingDragged()
{
	for(std::vector<TFControlPoint>::iterator iter = controlPoints.begin(); iter != controlPoints.end();)
	{
		TFControlPoint& controlPoint = *iter;

		bool dragging = controlPoint.isBeingDragged();

		if(dragging)
			return dragging;
		else
			++iter;
	}

	return false;
}

bool TransferFunctionGUI::isAnyControlPointSelected()
{
	for(std::vector<TFControlPoint>::iterator iter = controlPoints.begin(); iter != controlPoints.end();)
	{
		TFControlPoint& controlPoint = *iter;

		bool selected = controlPoint.isSelected();

		if(selected)
			return selected;
		else
			++iter;
	}

	return false;
}


TFControlPoint* TransferFunctionGUI::getSelectedControlPoint()
{
	for(std::vector<TFControlPoint>::iterator iter = controlPoints.begin(); iter != controlPoints.end();)
	{
		TFControlPoint& controlPoint = *iter;

		bool selected = controlPoint.isSelected();

		if(selected)
			return &controlPoint;
		else
			++iter;
	}

	return NULL;
}

bool TransferFunctionGUI::checkForControlPointsSelection(int x, int y)
{
	bool hit = false;

	for(std::vector<TFControlPoint>::iterator iter = controlPoints.begin(); iter != controlPoints.end();)
	{
		TFControlPoint& controlPoint = *iter;

		hit = AABBHitCheck(x, y, (float)TFControlPoint::DrawSize, controlPoint.m_pos.x * g_iTransferFuncPickerWidth, g_iTransferFuncPickerHeight - controlPoint.m_pos.y * g_iTransferFuncPickerHeight);
		if(hit)
		{
			controlPoint.setSelected(true);
			iter = controlPoints.end();
		}
		else
			++iter;
	}

	return hit;
}

void TransferFunctionGUI::processControlPointHitCheck(bool &hit, int x, int y, TFControlPoint &controlPoint)
{
	hit = AABBHitCheck(x, y, (float)TFControlPoint::DrawSize, controlPoint.m_pos.x * g_iTransferFuncPickerWidth, g_iTransferFuncPickerHeight - controlPoint.m_pos.y * g_iTransferFuncPickerHeight);
	if(hit)
	{
		controlPoint.setBeingDragged(true);
		setAllControlPointsUnselected();
		controlPoint.setSelected(true);
		controlPoint.dragMovement(x, y);
	}
}

bool TransferFunctionGUI::checkIfMouseInsideTransferFunc(int x, int y)
{
	return (x <= g_iTransferFuncPickerWidth  && x >= 0
		&& y <= g_iTransferFuncPickerHeight && y >= 0);
}

void TransferFunctionGUI::createNewControlPoint( int x, int y, const Lab_Colour& colour )
{
	float xRel = x / (float)g_iTransferFuncPickerWidth;
	float yRel = (g_iTransferFuncPickerHeight - y) / (float)g_iTransferFuncPickerHeight;

	setAllControlPointsUnselected();
	TFControlPoint newControlPoint = TFControlPoint(xRel, yRel, colour);
	newControlPoint.setSelected(true);
	controlPoints.push_back(newControlPoint);
}

void TransferFunctionGUI::deleteSelectedControlPoint()
{
	TFControlPoint* selectedControlPoint = getSelectedControlPoint();
	if(selectedControlPoint)
	{
		selectedControlPoint->setMarkedForDeletion(true);
		selectedControlPoint->setBeingDragged(true);
	}
	setAllControlPointsUndragged();

}

void TransferFunctionGUI::refreshTransferFuncPickerData()
{
	std::vector<TFControlPoint>::iterator currentLeft = controlPoints.begin();
	std::vector<TFControlPoint>::iterator currentRight = (controlPoints.begin() + 1);
	std::vector<TFControlPoint>::iterator controlPointsEnd = controlPoints.end();

	for(int x = 0; x < g_iTransferFuncPickerWidth; ++x)
	{

		RGB_Colour currentColour = RGB_Colour(0,0,0);

		//Check if our position is between the same ControlPoints as before. If no - we need the following ones
		while(currentRight != controlPointsEnd && x > currentRight->m_pos.x * g_iTransferFuncPickerWidth)
		{
			++currentRight;
			++currentLeft;
		}

		//Exceeding last right point
		if(currentRight == controlPointsEnd && x > currentLeft->m_pos.x * g_iTransferFuncPickerWidth)
		{
			currentColour = toRGB(currentLeft->getColourLab());
		}
		//Being before first left point
		else if(currentLeft != controlPointsEnd && x <= currentLeft->m_pos.x * g_iTransferFuncPickerWidth)
		{
			currentColour = toRGB(currentLeft->getColourLab());
		}//Normal
		else
		{
			float interPolBalance = calcInterpolationValueBetweenControlPoints(x, currentLeft, currentRight);
			currentColour = interpolateControlPointColours(interPolBalance, currentLeft, currentRight);
		}


		for(int y = 0; y < g_iTransferFuncPickerHeight; ++y)
		{
			GLubyte alpha = (GLubyte)(255 * y / (float)g_iTransferFuncPickerHeight);

			transferFunctionPickerData[(int)g_iTransferFuncPickerWidth * y + x] = RGBA_Colour(currentColour, alpha);
		}
	}
}



RGB_Colour TransferFunctionGUI::interpolateControlPointColours(float interPolBalance, std::vector<TFControlPoint>::iterator currentLeft, std::vector<TFControlPoint>::iterator currentRight)
{
	switch(transferFunctionTextureColorSpace)
	{
	case COLOURSPACE_RGB:
		{
			//Linear interpolation
			return linearInterpolationRGB(interPolBalance, toRGB(currentLeft->m_colour), toRGB(currentRight->m_colour));
		}
		break;
	case COLOURSPACE_LAB:
		{
			//Lab interpolation
			Lab_Colour& interpolatedLabColour = linearInterpolationLab(interPolBalance, currentLeft->m_colour, currentRight->m_colour);
			return toRGB(interpolatedLabColour);
		}	
		break;
	default:
		return RGB_Colour(0, 0, 0);
	}	
}

float TransferFunctionGUI::calcInterpolationValueBetweenControlPoints(int x, std::vector<TFControlPoint>::iterator currentLeft, std::vector<TFControlPoint>::iterator currentRight)
{
	float leftXAbs = currentLeft->m_pos.x * g_iTransferFuncPickerWidth;
	float rightXAbs = currentRight->m_pos.x * g_iTransferFuncPickerWidth;

	return (x - leftXAbs) / (rightXAbs - leftXAbs);	
}


void TransferFunctionGUI::updateControlPointDragMovement(int mouseX, int mouseY)
{
	for(std::vector<TFControlPoint>::iterator iter = controlPoints.begin(); iter != controlPoints.end(); ++iter)
	{
		TFControlPoint& controlPoint = *iter;

		if(controlPoint.isBeingDragged())
			controlPoint.dragMovement(mouseX, mouseY);
	}
}

bool TransferFunctionGUI::checkForControlPointsAction(int button, int state, int x, int y)
{
	bool hit = false;


	if(button == 2 && state == 0)
	{
		hit = checkForControlPointsSelection(x, y);

		if(hit)
		{
			deleteSelectedControlPoint();
		}
	}
	else if(button == 0 && state == 0)
	{
		TFControlPoint* selectedControlPoint = getSelectedControlPoint();

		if(selectedControlPoint != NULL)
		{
			processControlPointHitCheck(hit, x, y, *selectedControlPoint);
		}

		if(!hit)
		{

			for(std::vector<TFControlPoint>::iterator iter = controlPoints.begin(); iter != controlPoints.end();)
			{
				TFControlPoint& controlPoint = *iter;
				processControlPointHitCheck(hit, x, y, controlPoint);

				if(hit)
					iter = controlPoints.end();
				else
					++iter;
			}
		}
	}
	else if(button == 0 && state == 1)
	{
		bool dragging = isAnyControlPointBeingDragged();

		if(dragging)
		{
			setAllControlPointsUndragged();
		}
		else
		{
			setAllControlPointsUnselected();
			hit = checkForControlPointsSelection(x, y);
		}
	}
	return hit;
}


bool TransferFunctionGUI::checkForControlPointsCreation(int button, int state, int x, int y, const Lab_Colour& colour)
{
	bool mouseInsideTransferFuncPanel = checkIfMouseInsideTransferFunc(x, y);

	if(getSelectedControlPoint() == NULL)
	{
		if(button == 0 && state == 0 && mouseInsideTransferFuncPanel)
		{
			createNewControlPoint(x, y, colour);
		}
	}

	if(button == 2 && state == 0 && mouseInsideTransferFuncPanel)
		createNewControlPoint(x, y, colour);



	return mouseInsideTransferFuncPanel;
}



void TransferFunctionGUI::sortControlPoints()
{
	std::sort(controlPoints.begin(), controlPoints.end());
}

void TransferFunctionGUI::saveTransferFunctionControlPoints(const char* file)
{
	TiXmlDocument doc;

	TiXmlDeclaration * decl = new TiXmlDeclaration( "2.0beta", "", "" );
	doc.LinkEndChild( decl );

	for(std::vector<TFControlPoint>::iterator iter = controlPoints.begin(); iter != controlPoints.end(); ++iter)
	{
		TFControlPoint& controlPoint = *iter;
		std::stringstream stream;

		TiXmlElement* controlPointElement = new TiXmlElement( "ControlPoint" );
		doc.LinkEndChild(controlPointElement);

		TiXmlElement* xRel = new  TiXmlElement( "xRel" );
		TiXmlElement* yRel = new  TiXmlElement( "yRel" );
		TiXmlElement* colourL = new  TiXmlElement( "colourL" );
		TiXmlElement* colourA = new  TiXmlElement( "colourA" );
		TiXmlElement* colourB = new  TiXmlElement( "colourB" );

		controlPointElement->LinkEndChild(xRel);
		controlPointElement->LinkEndChild(yRel);
		controlPointElement->LinkEndChild(colourL);
		controlPointElement->LinkEndChild(colourA);
		controlPointElement->LinkEndChild(colourB);


		stream << controlPoint.m_pos.x;
		TiXmlText * xRelText = new TiXmlText(stream.str().c_str());
		stream.str("");
		xRel->LinkEndChild( xRelText );

		stream << controlPoint.m_pos.y;
		TiXmlText * yRelText = new TiXmlText(stream.str().c_str());
		stream.str("");
		yRel->LinkEndChild( yRelText );

		stream << controlPoint.getColourLab().L;
		TiXmlText * colourLText = new TiXmlText(stream.str().c_str());
		stream.str("");
		colourL->LinkEndChild( colourLText );

		stream << controlPoint.getColourLab().a;
		TiXmlText * colourAText = new TiXmlText(stream.str().c_str());
		stream.str("");
		colourA->LinkEndChild( colourAText );

		stream << controlPoint.getColourLab().b;
		TiXmlText * colourBText = new TiXmlText(stream.str().c_str());
		stream.str("");
		colourB->LinkEndChild( colourBText );
	}


	doc.SaveFile(file);
}


void TransferFunctionGUI::loadTransferFunctionControlPoints(std::string file)
{
	TiXmlDocument doc(file.c_str());

	bool loadOkay = doc.LoadFile();
	if (loadOkay)
	{
		controlPoints.clear();


		for (TiXmlNode* pChild = doc.FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
		{
			if(pChild->Type() == TiXmlNode::TINYXML_ELEMENT
				&&  std::string(pChild->Value()).compare("ControlPoint") == 0)
			{
				float xRel = 0.f;
				float yRel = 0.f;
				float colourL = 0.f;
				float colourA = 0.f;
				float colourB = 0.f;

				for (TiXmlNode* pChildAttribute = pChild->FirstChild(); pChildAttribute != 0; pChildAttribute = pChildAttribute->NextSibling())
				{
					if(pChildAttribute->Type() == TiXmlNode::TINYXML_ELEMENT)
					{
						if( std::string(pChildAttribute->Value()).compare("xRel") == 0)
							xRel = (float)atof(pChildAttribute->FirstChild()->Value());

						else if( std::string(pChildAttribute->Value()).compare("yRel") == 0)
							yRel = (float)atof(pChildAttribute->FirstChild()->Value());

						else if( std::string(pChildAttribute->Value()).compare("colourL") == 0)
							colourL = (float)atof(pChildAttribute->FirstChild()->Value());

						else if( std::string(pChildAttribute->Value()).compare("colourA") == 0)
							colourA = (float)atof(pChildAttribute->FirstChild()->Value());

						else if( std::string(pChildAttribute->Value()).compare("colourB") == 0)
							colourB = (float)atof(pChildAttribute->FirstChild()->Value());
					}
				}


				controlPoints.push_back(TFControlPoint(xRel, yRel, Lab_Colour(colourL, colourA, colourB)));
			}
		}
	}
	else
	{
		printf("Failed to load file \"%s\"\n", file);
	}

}