#ifndef TRANSFER_FUNCTION_GUI_H
#define TRANSFER_FUNCTION_GUI_H

#include "common.h"
#include <vector>

#include "Colours.h"

/**
* @class TransferFunctionGUI
*
* @brief  Handles transferfunction user interaction
*
*/


class TFControlPoint;

enum ColourSpace
{
	COLOURSPACE_RGB,
	COLOURSPACE_LAB
};

class TransferFunctionGUI
{
public:
	static float calcInterpolationValueBetweenControlPoints(int x, std::vector<TFControlPoint>::iterator currentLeft, std::vector<TFControlPoint>::iterator currentRight);
	static RGB_Colour interpolateControlPointColours(float interPolBalance, std::vector<TFControlPoint>::iterator currentLeft, std::vector<TFControlPoint>::iterator currentRight);


	static void refreshTransferFuncPicker();

	static void refreshTransferFuncPickerData();

	static void initControlPoints();

	static void sortControlPoints();
	static void drawControlPointsRelative();
	static void drawControlPointsConnection();

	static void saveTransferFunctionControlPoints(const char* file);
	static void loadTransferFunctionControlPoints(std::string file);

	static bool checkForControlPointsAction(int button, int state, int x, int y);

	static void deleteSelectedControlPoint();
	static bool checkForControlPointsSelection(int x, int y);
	static bool checkForControlPointsCreation(int button, int state, int x, int y, const Lab_Colour& colour);

	static void createNewControlPoint( int x, int y, const Lab_Colour& colour );
	static bool checkIfMouseInsideTransferFunc(int x, int y);
	static void processControlPointHitCheck(bool &hit, int x, int y, TFControlPoint &controlPoint);

	static bool isAnyControlPointBeingDragged();
	static bool isAnyControlPointSelected();

	static TFControlPoint* getSelectedControlPoint();

	static void setAllControlPointsUndragged();
	static void setAllControlPointsUnselected();

	static void updateControlPointDragMovement(int mouseX, int mouseY);

	//2D Transfer Functions display with control points
	static const int g_iTransferFuncPickerWidth;
	static const int g_iTransferFuncPickerHeight;

	static RGBA_Colour* transferFunctionPickerData;

	static std::vector<TFControlPoint> controlPoints;

	static int g_iTransferFunctionTextureDisplayHeight;
	static int g_iTransferFunctionTextureDisplayYOffset;

	static ColourSpace transferFunctionTextureColorSpace;

private:
	static bool AABBHitCheck(int mouseX, int mouseY, float AABBSideLength, float AABBCenterX, float AABBCenterY);
};

#endif