//Includes
#include "TransferFunctionTexture.h"
#include <algorithm>

#include "utility.h"
#include "Colours.h"

using namespace Colours;

//Static variables
GLuint TransferFunctionTexture::bottomWindowTransferFunctionTextureID = -1;
GLuint TransferFunctionTexture::renderWindowTransferFunctionTextureID = -1;

RGBA_Colour TransferFunctionTexture::transferFunctionTexture[transferFunctionTextureWidth];



void TransferFunctionTexture::initTransferFunctionTexture(int windowID, Window window)
{
	int initialWindow = glutGetWindow();
	glutSetWindow(windowID);

	GLuint* textureIDPointer = NULL;

	if(window == WINDOW_BOTTOM)
		textureIDPointer =  &bottomWindowTransferFunctionTextureID;
	if(window == WINDOW_RENDERING)
		textureIDPointer = &renderWindowTransferFunctionTextureID;


	if(textureIDPointer != NULL && windowID != GLuint(-1))
	{
		glGenTextures(1, textureIDPointer);

		get_errors();

		if(*textureIDPointer != GLuint(-1))
		{
			get_errors();
			glBindTexture(GL_TEXTURE_1D, *textureIDPointer);

			glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
			glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
			glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);

			glTexImage1D(GL_TEXTURE_1D, 0, GL_RGBA, transferFunctionTextureWidth, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);

			get_errors();
		}
	}

	glutSetWindow(initialWindow);
}


void TransferFunctionTexture::refreshTransferTextureDataWithControlPoints()
{
	std::vector<TFControlPoint>::iterator currentLeft = TransferFunctionGUI::controlPoints.begin();
	std::vector<TFControlPoint>::iterator currentRight = (TransferFunctionGUI::controlPoints.begin() + 1);
	std::vector<TFControlPoint>::iterator controlPointsEnd = TransferFunctionGUI::controlPoints.end();

	//Run through x values and interpolate colours
	for(int x = 0; x < transferFunctionTextureWidth; ++x)
	{
		//Recalculate x to fit the dimensions of the transferFuncPicker which is where the controlpoints are located on
		float xRel = (x  / (float) transferFunctionTextureWidth);

		//Check if our position is between the same ControlPoints as before. If no - we need the following ones
		while(currentRight != controlPointsEnd && xRel > currentRight->m_pos.x)
		{
			++currentRight;
			++currentLeft;
		}

		//Exceeding last right point
		if(currentRight == controlPointsEnd && xRel > currentLeft->m_pos.x)
		{
			float interPolBalance = (xRel - currentLeft->m_pos.x) / (float)(1.f - currentLeft->m_pos.x);
			GLubyte leftAlpha = (GLubyte)(255 * currentLeft->m_pos.y);
			GLubyte transparency = linearInterpolationAlpha
				(interPolBalance,
				leftAlpha,
				(GLubyte)0
				);
			transferFunctionTexture[x] = RGBA_Colour(toRGB(currentLeft->getColourLab()), transparency);
		}
		//Being before first left point
		else if((currentLeft != controlPointsEnd) && xRel < currentLeft->m_pos.x)
		{
			float interPolBalance = (currentLeft->m_pos.x - xRel) / currentLeft->m_pos.x;
			GLubyte leftAlpha = (GLubyte)(255 * currentLeft->m_pos.y);
			GLubyte transparency = linearInterpolationAlpha
				(interPolBalance,
				leftAlpha,
				(GLubyte)0
				);
			transferFunctionTexture[x] = RGBA_Colour(toRGB(currentLeft->getColourLab()), transparency);
		}//Normal
		else
		{
			float interPolBalance = TransferFunctionGUI::calcInterpolationValueBetweenControlPoints((int)(xRel * TransferFunctionGUI::g_iTransferFuncPickerWidth), currentLeft, currentRight);
			GLubyte transparency = linearInterpolationAlpha
				(interPolBalance,
				(GLubyte)(255 * currentLeft->m_pos.y),
				(GLubyte)(255 * currentRight->m_pos.y)
				);
			RGB_Colour& rgbResult = TransferFunctionGUI::interpolateControlPointColours(interPolBalance, currentLeft, currentRight);
			transferFunctionTexture[x].setColour(rgbResult, transparency);
		}
	}
}



void TransferFunctionTexture::refreshTransferFunctionTextureData()
{
	//Special case: No points
	if(TransferFunctionGUI::controlPoints.size() == 0)
	{
		for(int x = 0; x < transferFunctionTextureWidth; ++x)
			transferFunctionTexture[x] = RGBA_Colour((GLubyte)0, (GLubyte)0, (GLubyte)0, (GLubyte)0);
	}
	else
	{
		refreshTransferTextureDataWithControlPoints();
	}
}


void TransferFunctionTexture::copyDataToTransferFunctionTexture(int windowID, Window window)
{
	int initialWindow = glutGetWindow();
	glutSetWindow(windowID);

	GLuint* textureIDPointer = NULL;

	if(window == WINDOW_BOTTOM)
		textureIDPointer =  &bottomWindowTransferFunctionTextureID;
	if(window == WINDOW_RENDERING)
		textureIDPointer = &renderWindowTransferFunctionTextureID;

	if(textureIDPointer != NULL && windowID != GLuint(-1) && *textureIDPointer != GLuint(-1))
	{
		glBindTexture(GL_TEXTURE_1D, *textureIDPointer);
		glTexSubImage1D(GL_TEXTURE_1D, 0, 0, transferFunctionTextureWidth, GL_RGBA, GL_UNSIGNED_BYTE, transferFunctionTexture);
		get_errors();
	}

	glutSetWindow(initialWindow);
}
