#ifndef MAIN_H
#define MAIN_H


//Includes
#include "Arguments.h"
#include <string>
#include "Volume.h"
#include <il.h>

#include "BasicObjects.h"
#include "GlewContentManagement.h"

class GLUI_Listbox;
class GLUI_Scrollbar;
class GLUI_Spinner;
class GLUI_StaticText;
class GLUI_RadioGroup;
class GLUI_Rotation;

class Shader;

class HistogramDraw;

//Namespaces
using namespace std;

//Global functions

int initialization( int argc, char ** argv);
void initGlutMainWindow( int argc, char ** argv );
int initDevil();
void initGLUIMaster();
void initRightWindow();

void initGlutSubWindowRendering();
void initGlutSubWindowBottom();
void initGlutSubWindowRight();

void initInterfaceBottomValues();

bool initShaders();

void initBottomWindow();
void initRenderWindow();

void enterMainLoop();

void processSliceOfData(std::string strDataFile );

void positionRenderWindow(int width, int height);
void positionRightWindow(int width, int height);
void positionBottomWindow(int width, int height);

void reshapeRenderWindow(int width, int height);
void reshapeRightWindow(int width, int height);
void reshapeBottomWindow(int width, int height);

void saveSliceTo2DTexture();
void saveDataTo3DTexture();

void gluiMasterSpecial(int a, int b, int c);
void glutKeyboard(unsigned char key, int x, int y);
void mainWindowMouseCallback(int button, int state, int x, int y);
void mouseWheelFuncRenderWindow(int, int, int, int);

void mouseMotionRenderWindow(int x, int y);
void mouseMotionBottomWindow(int x, int y);

void reshapeFuncBottomWindow(int width, int height);
void reshapeFuncRenderWindow(int width, int height);
void reshapeFuncRightWindow(int width, int height);

void orthogonalStart(int width, int height);
void orthogonalEnd();

void handleMouseActionBottomWindow(int button, int state, int x, int y);
void glutBottomWindowKeyboard(unsigned char key, int x, int y);

void drawTransferTexture(int windowWidth, int windowHeight, float xOffset, float yOffset);
void drawControlPoints(int windowWidth, int windowHeight, int xOffset, int yOffset);
void drawColourPicker(int windowWidth, int windowHeight, int xOffset, int yOffset);
void drawColourPickerCursor(int windowWidth, int windowHeight, int xOffset, int yOffset);
void drawHistogram(int windowWidth, int windowHeight, int xOffset, int yOffset);

void initInterfaceBottom();

void initInterfaceRight();

void handleColourSpaceListboxChange(int id);
void openFile(int);

void updateRenderWindow(int);
void updateBottomWindow(int);
void renderBottomWindow();
void renderRightWindow();
void renderRenderWindow();

void initTransferFunctionBackgroundTexture();
void deinitTransferFunctionBackgroundTexture();

void drawTransferFuncPickerBackground(int windowWidth, int windowHeight, int xOffset, int yOffset);
void drawTransferFuncPickerForeground( int xOffset, int yOffset );
void drawTransferFuncPicker(int windowWidth, int windowHeight, int transferFuncXOffset, int transferFuncYOffset);

bool checkForColourPickerAction(int button, int state, int x, int y);

void handleColourPickerSliderModeChange(int id);
void handleColourPickerSliderChange(int id);
void handleHistoGramDrawTypeListboxChange(int id);

void mouseActionBottomWindow(int button, int state, int x, int y);
void mouseActionRenderWindow(int button, int state, int x, int y);

void updateGUIColourWithColourPickerValue();

void refreshColourMapCornerPositions();

void updateColourPickerCursorMovement( int x, int y );

void updateColourPickerCursorPosBasedOnMouseCoordinates( int x, int y );

void transformCoordinatesToTransferFunctionPickerSpace(int &transferFuncSpaceX, int &transferFuncSpaceY);

void updateGUIColourWithSelectedControlPointColour();
void updateSelectedControlPointColourWithGUIColour();
void updateColourPickerCursorPosBasedOnSelectedColour();

void refreshBottomWindowData();

void handleGUIColourLabLChange(int);
void handleGUIColourLabAChange(int);
void handleGUIColourLabBChange(int);

void handleGUIColourLabChange(int);
void handleGUIColourRgbChange(int);
void handleGUISpecifiedColourChange(int);

void handleSaveTransferFunctionControlPoints(int);
void handleLoadTransferFunctionControlPoints(int);

void postRedisplayBottomWindow();

void prepareRenderWindowRendering();

void updateColourPickerSliderWithSetGUIColourComponent();

void drawBoundingBox();

void drawBox(float cubeWidth, float cubeHeight, float cubeDepth);
void drawCenterCross();
void drawVolume(float cubeWidth, float cubeHeight, float cubeDepth);

void startShading();
void stopShading();

void renderToFullscreenQuad();

void initFBOs();
void refreshFBOs();
void refreshFBOTextureSizes();

void startRenderViewPerspective();
void startFBOPerspective();

void refreshShaderData();


void refreshColourPickerAfterSliderChange();

void startRatioFixScale();
void endRatioFixScale();

void updateViewRotationWithDragPosition(int x, int y);

//Global variables
int g_iMainWindowWidth = 1280, g_iMainWindowHeight = 720; // window dimensions
int g_iRenderWindowWidth = 0, g_iRenderWindowHeight = 0; // render viewport
int g_rightWindowWidth = 180, g_bottomWindowHeight = 220;
Arguments g_argArguments; // arguments that are parsed from the console
Volume g_Volume; // our volume
unsigned int g_uExampleTexture = 0; // texture

float g_fCoordXYZ = 0.5f; // x|y|z-Coordinate of the currently displayed slice (only for 3D textures)
float g_fIncrement = 0.f; // increment for the z-texture coordinates (only for 3D textures)
int g_iSampleRate; // sample rate
float rotMat[16] = { 1.0, 0.0, 0.0, 0.0,		0.0, 1.0, 0.0, 0.0, 0.0,	0.0, 1.0, 0.0,		0.0, 0.0, 0.0, 1.0}; //rotation matrix
float *g_pRotate = rotMat; // pointer to rotation matrix
float g_fRatio_rw = 0.f; // ratio of render window
float g_fRatio_xy = 0.f; // ratio of width to height of 3D texture
float g_fRatio_xz = 0.f; // ratio of width to depth of 3D texture 
float g_fRatio_yz = 0.f; // ratio of height to depth of 3D texture

int g_iSliceDir = 0;
int g_iPerspectiveType = 0;
int g_iCombinationType = 0;

HWND g_hMainWindow;
int g_mainWindowID		=	0;
int g_renderWindowID	=	0;
int g_bottomWindowID	=	0;
int g_rightWindowID		=	0;

GLUI_Listbox*		colourSpaceListbox;
GLUI_Scrollbar*		g_pGLUI_SliderSlice;
GLUI_Scrollbar*		g_pGLUI_SliderSampleRate;
GLUI_Spinner*		g_pGLUI_SpinnerSampleRate;
GLUI_Scrollbar*		g_pColourPickerSlider;
GLUI_StaticText*	g_pColourPickerSliderText;

int g_iTransferFunctionPickerXOffset = 5;
int g_iTransferFunctionPickerYOffset = 28;

GLuint transferFunctionBackgroundTexture;
ILuint transferFunctionBackgroundImage;

int g_iTransferFunctionBackgroundTextureWidth;
int g_iTransferFunctionBackgroundTextureHeight;

GLUI_RadioGroup* LabSwitcherForColourPicker;

int g_iColourPicker_xBegin, g_iColourPicker_xEnd, g_iColourPicker_yBegin, g_iColourPicker_yEnd;

float g_ColorPickerCursorXPos = 0.f;
float g_ColorPickerCursorYPos = 0.f;

bool g_bDraggingColourPickerCursor = false;

GLUI_Spinner* g_ColourSpinnerRgbR, * g_ColourSpinnerRgbG, * g_ColourSpinnerRgbB;
int g_iColourRgbR, g_iColourRgbG, g_iColourRgbB;

GLUI_Spinner* g_ColourSpinnerLabL, * g_ColourSpinnerLabA, * g_ColourSpinnerLabB;
float g_fColourLabL, g_fColourLabA, g_fColourLabB;

GLUI_Spinner* g_ColourTransparencySpinner;
int g_iColourAlpha;

HistogramDraw*		g_HistogramDraw;

GLUI_Listbox*		g_histogramDrawTypeListbox;

static const unsigned int		simpleShaderID = 0;

GLuint	g_uBackBufferTexture = 0;
GLuint	g_uFrontBufferTexture = 0;

float zoom = 0.0;

GLUI_Rotation *g_viewRotationGLUI = 0;

float g_fCubeDimensionNormedX, g_fCubeDimensionNormedY, g_fCubeDimensionNormedZ;

GLuint g_uBackBufferFBO;
GLuint g_uFrontBufferFBO;

Shader* shader;

GLint front2DTexLoc;
GLint back2DTexLoc;
GLint data3DTexLoc;
GLint transferTexLoc;

GLint slicesLoc;
GLint combTypeLoc;
GLint sampleRateLoc;
GLint sliceDepthLoc;

int viewRotationDragStartX = 0;
int viewRotationDragStartY = 0;
bool viewRotationDragging = false;



#endif