#include "utility.h"
#include <fstream>

#include <iostream>

#include "common.h"

bool file_exists(const string &filename)
{
	std::ifstream ifile(filename.c_str());
	return ifile.good();
}

string read_file(const string &filename)
{
	std::ifstream ifile(filename.c_str());

	return string(std::istreambuf_iterator<char>(ifile),
		std::istreambuf_iterator<char>());
}

void my_get_errors(const char *location)
{
	GLenum error = glGetError();

	if (error != GL_NO_ERROR) {
		cerr << "Error at " << location << ": "; 
		switch (error) {
		case GL_INVALID_ENUM:
			cerr << "GL: enum argument out of range." << endl;
			break;
		case GL_INVALID_VALUE:
			cerr << "GL: Numeric argument out of range." << endl;
			break;
		case GL_INVALID_OPERATION:
			cerr << "GL: Operation illegal in current state." << endl;
			break;
		case GL_STACK_OVERFLOW:
			cerr << "GL: Stack overflow." << endl;
			break;
		case GL_STACK_UNDERFLOW:
			cerr << "GL: Stack underflow." << endl;
			break;
		case GL_OUT_OF_MEMORY:
			cerr << "GL: Not enough memory left to execute command." << endl;
			break;
		default:
			cerr << "GL: Unknown error." << endl;
		}
	}
}
